@@ -19,6 +19,7 @@ package cl
 
 import (
 	"context"
+	"fmt"
 	"go/types"
 	"log"
 	"os"
@@ -124,8 +125,22 @@ type nodeInterp struct {
 	workingDir string
 }
 
-func (p *nodeInterp) Position(pos token.Pos) token.Position {
-	return p.fset.Position(pos)
+func (p *nodeInterp) Position(start token.Pos) token.Position {
+	pos := p.fset.Position(start)
+	pos.Filename = relFile(p.workingDir, pos.Filename)
+	return pos
+}
+
+func (p *nodeInterp) Caller(node ast.Node) string {
+	if expr, ok := node.(*ast.CallExpr); ok {
+		node = expr.Fun
+		start := node.Pos()
+		pos := p.fset.Position(start)
+		f := p.files[pos.Filename]
+		n := int(node.End() - start)
+		return string(f.Code[pos.Offset : pos.Offset+n])
+	}
+	return "the function call"
 }
 
 func (p *nodeInterp) LoadExpr(node ast.Node) (src string, pos token.Position) {
@@ -153,7 +168,8 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 	if targetDir == "" {
 		targetDir = dir
 	}
-	ctx := &pkgCtx{syms: make(map[string]loader)}
+	interp := &nodeInterp{fset: conf.Fset, files: pkg.Files, workingDir: workingDir}
+	ctx := &pkgCtx{syms: make(map[string]loader), nodeInterp: interp}
 	loadUnderlying := func(at *gox.Package, typ *types.Named) types.Type {
 		return doLoadUnderlying(ctx, typ)
 	}
@@ -167,7 +183,7 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 		LoadPkgs:        conf.PkgsLoader.LoadPkgs,
 		LoadUnderlying:  loadUnderlying,
 		HandleErr:       ctx.handleErr,
-		NodeInterpreter: &nodeInterp{fset: conf.Fset, files: pkg.Files, workingDir: workingDir},
+		NodeInterpreter: interp,
 		Prefix:          gopPrefix,
 		ParseFile:       nil, // TODO
 		NewBuiltin:      newBuiltinDefault,
@@ -181,7 +197,16 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 		for _, decl := range f.Decls {
 			switch d := decl.(type) {
 			case *ast.FuncDecl:
-				ctx.loadSymbol(d.Name.Name)
+				if d.Recv == nil {
+					name := d.Name.Name
+					if name != "init" {
+						ctx.loadSymbol(name)
+					}
+				} else {
+					if name, ok := getRecvTypeName(ctx, d.Recv, false); ok {
+						getTypeLoader(ctx.syms, token.NoPos, name).load()
+					}
+				}
 			case *ast.GenDecl:
 				switch d.Tok {
 				case token.TYPE:
@@ -207,39 +232,68 @@ func NewPackage(pkgPath string, pkg *ast.Package, conf *Config) (p *gox.Package,
 
 type loader interface {
 	load()
+	pos() token.Pos
 }
 
-type loaderFunc func()
+type baseLoader struct {
+	fn    func()
+	start token.Pos
+}
 
-func (fn loaderFunc) load() {
-	fn()
+func initLoader(ctx *pkgCtx, syms map[string]loader, start token.Pos, name string, fn func()) {
+	if old, ok := syms[name]; ok && start != token.NoPos {
+		pos := ctx.Position(start)
+		oldpos := ctx.Position(old.pos())
+		ctx.handleCodeErrorf(
+			&pos, "%s redeclared in this block\n\tprevious declaration at %v", name, oldpos)
+	}
+	syms[name] = &baseLoader{start: start, fn: fn}
+}
+
+func (p *baseLoader) load() {
+	p.fn()
+}
+
+func (p *baseLoader) pos() token.Pos {
+	return p.start
 }
 
 type typeLoader struct {
 	typ, typInit func()
 	methods      []func()
+	start        token.Pos
 }
 
-func getTypeLoader(syms map[string]loader, name string) *typeLoader {
+func getTypeLoader(syms map[string]loader, start token.Pos, name string) *typeLoader {
 	t, ok := syms[name]
-	if !ok {
-		t = &typeLoader{}
+	if ok {
+		if start != token.NoPos {
+			panic("TODO: redefine")
+		}
+	} else {
+		t = &typeLoader{start: start}
 		syms[name] = t
 	}
 	return t.(*typeLoader)
 }
 
+func (p *typeLoader) pos() token.Pos {
+	return p.start
+}
+
 func (p *typeLoader) load() {
 	doNewType(p)
 	doInitType(p)
 	doInitMethods(p)
 }
 
-func doNewType(ld *typeLoader) {
+func doNewType(ld *typeLoader) bool {
 	if typ := ld.typ; typ != nil {
 		ld.typ = nil
 		typ()
+		return true
 	}
+	return false
 }
 
 func doInitType(ld *typeLoader) {
@@ -271,6 +325,7 @@ func (p *Errors) Error() string {
 }
 
 type pkgCtx struct {
+	*nodeInterp
 	syms  map[string]loader
 	inits []func()
 	errs  []error
@@ -286,6 +341,14 @@ type blockCtx struct {
 	imports   map[string]*gox.PkgRef
 }
 
+func newCodeErrorf(pos *token.Position, format string, args ...interface{}) *gox.CodeError {
+	return &gox.CodeError{Pos: pos, Msg: fmt.Sprintf(format, args...)}
+}
+
+func (p *pkgCtx) handleCodeErrorf(pos *token.Position, format string, args ...interface{}) {
+	p.handleErr(newCodeErrorf(pos, format, args...))
+}
+
 func (p *pkgCtx) handleErr(err error) {
 	p.errs = append(p.errs, err)
 }
@@ -308,8 +371,7 @@ func (p *pkgCtx) loadType(name string) {
 func (p *pkgCtx) loadSymbol(name string) bool {
 	if f, ok := p.syms[name]; ok {
 		if ld, ok := f.(*typeLoader); ok {
-			doNewType(ld) // create this type, but don't init
-			return true
+			return doNewType(ld) // create this type, but don't init
 		}
 		delete(p.syms, name)
 		f.load()
@@ -328,23 +390,24 @@ func loadFile(p *gox.Package, parent *pkgCtx, f *ast.File, targetDir string, fil
 		switch d := decl.(type) {
 		case *ast.FuncDecl:
 			if d.Recv == nil {
-				name := d.Name.Name
+				name := d.Name
 				fn := func() {
 					loadFunc(ctx, nil, d)
 				}
-				if name == "init" {
+				if name.Name == "init" {
 					parent.inits = append(parent.inits, fn)
 				} else {
-					syms[name] = loaderFunc(fn)
+					initLoader(parent, syms, name.Pos(), name.Name, fn)
 				}
 			} else {
-				name := getRecvTypeName(d.Recv)
-				ld := getTypeLoader(syms, name)
-				ld.methods = append(ld.methods, func() {
-					doInitType(ld)
-					recv := toRecv(ctx, d.Recv)
-					loadFunc(ctx, recv, d)
-				})
+				if name, ok := getRecvTypeName(parent, d.Recv, true); ok {
+					ld := getTypeLoader(syms, token.NoPos, name)
+					ld.methods = append(ld.methods, func() {
+						doInitType(ld)
+						recv := toRecv(ctx, d.Recv)
+						loadFunc(ctx, recv, d)
+					})
+				}
 			}
 		case *ast.GenDecl:
 			switch d.Tok {
@@ -356,7 +419,7 @@ func loadFile(p *gox.Package, parent *pkgCtx, f *ast.File, targetDir string, fil
 				for _, spec := range d.Specs {
 					t := spec.(*ast.TypeSpec)
 					name := t.Name.Name
-					ld := getTypeLoader(syms, name)
+					ld := getTypeLoader(syms, t.Name.Pos(), name)
 					ld.typ = func() {
 						if t.Assign != token.NoPos { // alias type
 							ctx.pkg.AliasType(name, toType(ctx, t.Type))
@@ -370,20 +433,26 @@ func loadFile(p *gox.Package, parent *pkgCtx, f *ast.File, targetDir string, fil
 				}
 			case token.CONST:
 				for _, spec := range d.Specs {
-					v := spec.(*ast.ValueSpec)
-					names := makeNames(v.Names)
-					setNamesLoader(syms, names, func() {
-						loadConsts(ctx, names, v)
-						removeNames(syms, names)
+					vSpec := spec.(*ast.ValueSpec)
+					setNamesLoader(parent, syms, vSpec.Names, func() {
+						if v := vSpec; v != nil { // only init once
+							vSpec = nil
+							names := makeNames(v.Names)
+							loadConsts(ctx, names, v)
+							removeNames(syms, names)
+						}
 					})
 				}
 			case token.VAR:
 				for _, spec := range d.Specs {
-					v := spec.(*ast.ValueSpec)
-					names := makeNames(v.Names)
-					setNamesLoader(syms, names, func() {
-						loadVars(ctx, names, v)
-						removeNames(syms, names)
+					vSpec := spec.(*ast.ValueSpec)
+					setNamesLoader(parent, syms, vSpec.Names, func() {
+						if v := vSpec; v != nil { // only init once
+							vSpec = nil
+							names := makeNames(v.Names)
+							loadVars(ctx, names, v)
+							removeNames(syms, names)
+						}
 					})
 				}
 			default:
@@ -406,7 +475,13 @@ func loadType(ctx *blockCtx, t *ast.TypeSpec) {
 
 func loadFunc(ctx *blockCtx, recv *types.Var, d *ast.FuncDecl) {
 	sig := toFuncType(ctx, d.Type, recv)
-	fn := ctx.pkg.NewFuncWith(d.Pos(), d.Name.Name, sig)
+	fn, err := ctx.pkg.NewFuncWith(d.Pos(), d.Name.Name, sig, func() token.Pos {
+		return d.Recv.List[0].Type.Pos()
+	})
+	if err != nil {
+		ctx.handleErr(err)
+		return
+	}
 	if body := d.Body; body != nil {
 		loadFuncBody(ctx, fn, body)
 	}
@@ -435,7 +510,7 @@ func loadConsts(ctx *blockCtx, names []string, v *ast.ValueSpec) {
 	if v.Type != nil {
 		typ = toType(ctx, v.Type)
 	}
-	cb := ctx.pkg.NewConstStart(typ, names...)
+	cb := ctx.pkg.NewConstStart(v.Names[0].Pos(), typ, names...)
 	for _, val := range v.Values {
 		compileExpr(ctx, val)
 	}
@@ -447,7 +522,7 @@ func loadVars(ctx *blockCtx, names []string, v *ast.ValueSpec) {
 	if v.Type != nil {
 		typ = toType(ctx, v.Type)
 	}
-	varDecl := ctx.pkg.NewVar(typ, names...)
+	varDecl := ctx.pkg.NewVar(v.Names[0].Pos(), typ, names...)
 	if v.Values != nil {
 		cb := varDecl.InitStart(ctx.pkg)
 		for _, val := range v.Values {
@@ -471,9 +546,9 @@ func removeNames(syms map[string]loader, names []string) {
 	}
 }
 
-func setNamesLoader(syms map[string]loader, names []string, load func()) {
+func setNamesLoader(ctx *pkgCtx, syms map[string]loader, names []*ast.Ident, load func()) {
 	for _, name := range names {
-		syms[name] = loaderFunc(load)
+		initLoader(ctx, syms, name.Pos(), name.Name, load)
 	}
 }
 
