@@ -34,7 +34,7 @@ import (
 func compileExprLHS(ctx *blockCtx, expr ast.Expr) {
 	switch v := expr.(type) {
 	case *ast.Ident:
-		compileIdentLHS(ctx, v.Name)
+		compileIdentLHS(ctx, v)
 	case *ast.IndexExpr:
 		compileIndexExprLHS(ctx, v)
 	case *ast.SelectorExpr:
@@ -123,7 +123,7 @@ func compileBinaryExpr(ctx *blockCtx, v *ast.BinaryExpr) {
 func compileIndexExprLHS(ctx *blockCtx, v *ast.IndexExpr) {
 	compileExpr(ctx, v.X)
 	compileExpr(ctx, v.Index)
-	ctx.cb.IndexRef(1)
+	ctx.cb.IndexRef(1, v)
 }
 
 func compileStarExprLHS(ctx *blockCtx, v *ast.StarExpr) { // *x = ...
@@ -148,7 +148,7 @@ func compileTypeAssertExpr(ctx *blockCtx, v *ast.TypeAssertExpr, twoValue bool)
 func compileIndexExpr(ctx *blockCtx, v *ast.IndexExpr, twoValue bool) { // x[i]
 	compileExpr(ctx, v.X)
 	compileExpr(ctx, v.Index)
-	ctx.cb.Index(1, twoValue)
+	ctx.cb.Index(1, twoValue, v)
 }
 
 func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
@@ -158,7 +158,7 @@ func compileSliceExpr(ctx *blockCtx, v *ast.SliceExpr) { // x[i:j:k]
 	if v.Slice3 {
 		compileExprOrNone(ctx, v.Max)
 	}
-	ctx.cb.Slice(v.Slice3)
+	ctx.cb.Slice(v.Slice3, v)
 }
 
 func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
@@ -176,7 +176,7 @@ func compileSelectorExprLHS(ctx *blockCtx, v *ast.SelectorExpr) {
 	default:
 		compileExpr(ctx, v.X)
 	}
-	ctx.cb.MemberRef(v.Sel.Name)
+	ctx.cb.MemberRef(v.Sel.Name, v)
 }
 
 func compileSelectorExpr(ctx *blockCtx, v *ast.SelectorExpr, autoCall bool) {
@@ -227,7 +227,7 @@ func compileCallExpr(ctx *blockCtx, v *ast.CallExpr) {
 	for _, arg := range v.Args {
 		compileExpr(ctx, arg)
 	}
-	ctx.cb.Call(len(v.Args), v.Ellipsis != gotoken.NoPos)
+	ctx.cb.CallWith(len(v.Args), v.Ellipsis != gotoken.NoPos, v)
 }
 
 func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
@@ -241,7 +241,8 @@ func compileFuncLit(ctx *blockCtx, v *ast.FuncLit) {
 	}
 }
 
-func compileIdentLHS(ctx *blockCtx, name string) {
+func compileIdentLHS(ctx *blockCtx, v *ast.Ident) {
+	name := v.Name
 	if name == "_" {
 		ctx.cb.VarRef(nil)
 	} else {
@@ -249,7 +250,7 @@ func compileIdentLHS(ctx *blockCtx, name string) {
 		if o == nil {
 			panic("TODO: var not found")
 		}
-		ctx.cb.VarRef(o)
+		ctx.cb.VarRef(o, v)
 	}
 }
 
@@ -413,7 +414,7 @@ func compileComprehensionExpr(ctx *blockCtx, v *ast.ComprehensionExpr, twoValue
 	ret := pkg.NewAutoParam("_gop_ret")
 	var results *types.Tuple
 	if kind == comprehensionSelect && twoValue {
-		boolean := pkg.NewParam("_gop_ok", types.Typ[types.Bool])
+		boolean := pkg.NewParam(token.NoPos, "_gop_ok", types.Typ[types.Bool])
 		results = types.NewTuple(ret, boolean)
 	} else {
 		results = types.NewTuple(ret)
